# https://engineeringblog.yelp.com/2017/06/making-photos-smaller.html


def jpeg_minify(jpeg, settings: dict):
    """Minify JPEG main function."""
    # TODO: Implement this
    # print(f"JPEG WAS CALLED FOR {len(jpeg)}!!!!! #############################################")
    return jpeg
