# https://engineeringblog.yelp.com/2017/06/making-photos-smaller.html


# import pngquant


def png_minify(png, settings: dict):
    """Minify PNG main function."""
    #    pngquant.config()
    #    png=quant_data(self, data=None, dst=None, ndeep=None, ndigits=None, delete=True):
    # TODO: Implement this
    return png
